#!/usr/bin/perl
## ######### PROJECT NAME : ##########
## 
## exec_cmd_object.pm for aspiq plugins
## 
## ######### PROJECT DESCRIPTION : ###
## 
## This file is an aspiq object. It allows user to
##exec command with root rights.
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
## 
## Started on  Wed Jan 30 11:50:26 2008 Boutonnet Alexandre
## Last update Wed Oct 29 18:10:14 2008 Boutonnet Alexandre
## 
## ###################################
##

package exec_cmd;

use strict;
use warnings;

use IPC::Open3;
use Data::Dumper;
use POSIX ":sys_wait_h";

# Error tab
my $error_hash = {
    0 => 'Success',
    1 => 'Unable to write data to the child process, it exited before writing',
    100 => 'Internal error',
    101 => 'Line parameter not found',
    200 => 'Read only mode : this command can not be executed',
    300 => 'This method does nothing in this plugin.',
};


# This function is the constructor of this object
# The parameter of this method is an instance of the
# log object.
sub new
{
    my $log_obj = shift; # Getting the log object.

    my $OBJ = {}; # This had to contain object's personnal datas.

    $OBJ->{log} = $log_obj;

    bless($OBJ);

    return($OBJ);
}

# This function permit to user to initiate this object
# The parameter of this method is the read-only boolean
sub init
{
    my ($OBJ, $read_only) = @_;

    $OBJ->{read_only} = $read_only;

    return(0);
}

# This method permit to user to get info
# The parameter is an hash containing parameters.
sub get
{
    return(_make_error_hash(300, $error_hash->{300}));
}

# This method permit to user to make an action
# The parameter is an hash containing parameters.
sub set
{
    my ($OBJ, $hash) = @_;

    # Testing if hash contains line key
    return(_make_error_hash(101, $error_hash->{101})) if (!exists($hash->{'line'}));

    local(*READ, *WRITE, *ERROR);

    # Executing my command
    # Whe assume that $hash->{line} is not empty
    my $pid = open3( *WRITE, *READ, *ERROR, $hash->{'line'}[0]);

    # If datas have to been written..
    if (exists($hash->{'write_data'}))
    {
	# Testing if child process is still alive
	waitpid($pid, &WNOHANG);
	if ($? != -1)
	{
	    return(_make_error_hash(1, $error_hash->{1}));
	}

	# Writing datas !
	my $data = $hash->{'write_data'}[0];
	print WRITE $data if (-w WRITE);
    }
    close(WRITE);

    # Reading standard output
    my $read_buffer = '';
    while(<READ>)
    {
	$read_buffer .= $_;
    }
    close(READ);

    # Reading error output
    my $err_buffer = '';
    while(<ERROR>)
    {
	$err_buffer .= $_;
    }
    close(ERROR);

    # waiting for my child
    waitpid($pid, 0);
    
    # getting child return value
    my $return_value = $?;

    # Building my hash
    my $hash_return = {
	'stdout' => [
		     $read_buffer,
		     ],
	    'stderr' => [
			 $err_buffer,
			 ],
	    'return_value' => [
			       $return_value,
			       ],
	};

    return($hash_return);
}

# This method permit to the user to get all info
# There is no parameter to this method.
sub info
{
    return(_make_error_hash(300, $error_hash->{300}));
}

# This method permit to the user to delete an entity.
# The parameter is an hash containing parameters.
sub delete
{
    return(_make_error_hash(300, $error_hash->{300}));
}

# This method permit to the user to get all available method
# There is no parameter.
sub list_methods
{
    my $OBJ = shift;

    my $hash_return = {
	'methods' => [
		      'list_error_codes',
		      ],
	};
    
    push(@{$hash_return->{'methods'}}, "set") if(!$OBJ->{read_only});

    return($hash_return);
}


# This method permit to the user to get all error codes.
# There is no parameter.
sub list_error_codes
{
    my $hash_return = {
	'error_codes' => []
    };

    foreach my $key (sort keys(%{$error_hash}))
    {
	my $tmp_hash = {
	    'error' => [$key],
	    'msg' => [$error_hash->{$key}],
	};
	push(@{$hash_return->{'error_codes'}}, $tmp_hash);
    }

    return($hash_return);
}

### Private methods
sub _make_error_hash
{
    my ($error_code, $error_msg) = @_;

    my $hash_error = {
	'error_code' => [$error_code],
	'error_msg' => [$error_msg]
	};

    return($hash_error);
}

1;
