#!/usr/bin/perl -w
## ######### PROJECT NAME : ##########
## 
## network_object.pm for aspiq_core
## 
## ######### PROJECT DESCRIPTION : ###
## 
## This lib permits to the aspiq core to handle network interfaces
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
## 
## Started on  Thu Mar  6 15:08:58 2008 Boutonnet Alexandre
## Last update Mon Mar 17 18:05:47 2008 Boutonnet Alexandre
## 
## ###################################
##

package network;

use strict;
use warnings;

use Data::Dumper;

use Lib_Network;

# Error tab
my $error_hash = {
    0 => 'Success',
    100 => 'Internal error',
    101 => 'Iface parameter not found',
    102 => 'Interface name not found',
    103 => 'Interface not found : ',
    104 => 'An error occured : ',
    105 => 'No iface to delete. Please ensure that this iface exists.',
    106 => 'Unable to delete iface in config file : ',
    200 => 'Read only mode : this command can not be executed',
    300 => 'This method does nothing in this plugin',
};


# This function is the constructor of this object
# The parameter of this method is an instance of the
# log object.
sub new
{
    my $log_obj = shift; # Getting the log object.

    my $OBJ = {}; # This had to contain object's personnal datas.

    $OBJ->{log} = $log_obj;

    bless($OBJ);

    return($OBJ);
}

# This function permit to user to initiate this object
# The parameter of this method is the read-only boolean
sub init
{
    my ($OBJ, $read_only) = @_;

    $OBJ->{read_only} = $read_only;

    return(0);
}

# This method permit to user to get info
# The parameter is an hash containing parameters.
sub get
{
    my ($OBJ, $param_hash) = @_;

    # Testing if hash contains iface key
    return(_make_error_hash(101, $error_hash->{101})) if (!exists($param_hash->{'iface'}));

    my $hash_iface = Lib_Network::get_ifaces_info();

    my $hash_return = {};
    foreach my $iface (@{$param_hash->{'iface'}})
    {
	# Testing if hash contains name key
	return(_make_error_hash(102, $error_hash->{102})) if (!exists($iface->{'name'}));

	# This variable contains the iface name
	my $iface_name = $iface->{'name'}[0];

	return (_make_error_hash(103, $error_hash->{103}.$iface_name)) if (!exists($hash_iface->{$iface_name}));
	
	$hash_return->{$iface_name} = $hash_iface->{$iface_name};
    }

    return (_make_hash($hash_return));
}

# This method permit to user to make an action
# The parameter is an hash containing parameters.
sub set
{
    my ($OBJ, $param_hash) = @_;

    # Testing the read only mode..
    return(_make_error_hash(200, $error_hash->{200})) if ($OBJ->{read_only});

    # Testing if hash contains iface key
    return(_make_error_hash(101, $error_hash->{101})) if (!exists($param_hash->{'iface'}));

    # Translating hash..
    my $usable_hash = _unmake_hash($param_hash);

    # Setting interfaces ..
    my ($err, $msg) = Lib_Network::set_ifaces($usable_hash);

    return (_make_error_hash(104, $error_hash->{104}.$msg)) if ($err);

    return (_make_error_hash(0, $error_hash->{0}));
}

# This method permit to the user to get all info
# There is no parameter to this method.
sub info
{
    my $net_hash = _make_hash(Lib_Network::get_ifaces_info());

    return(_make_error_hash(100, $error_hash->{100})) if (!$net_hash);

    return ($net_hash);
}

# This method permit to the user to delete an entity.
# The parameter is an hash containing parameters.
sub delete
{
    my ($OBJ, $param_hash) = @_;

    # Testing the read only mode..
    return(_make_error_hash(200, $error_hash->{200})) if ($OBJ->{read_only});

    # Getting ifaces infos..
    my $net_hash = Lib_Network::get_ifaces_info();

    # Flag for testing if there is a iface to delete..
    my $flag = 0;

    # Searching for iface to delete
    foreach my $tmp_hash (@{$param_hash->{'iface'}})
    {
	# If my iface to delete is in the hash ..
	if (exists($net_hash->{$tmp_hash->{'name'}[0]}))
	{
	    # I ifdown the interface.
	    my ($err, $msg) = Lib_Network::ifdown($tmp_hash->{'name'}[0]);
	    return (_make_error_hash(104, $error_hash->{104}.$msg)) if ($err);

	    # I delete the interface in the config file
	    my ($err, $msg) = Lib_Network::delete_iface($tmp_hash->{'name'}[0]);

	    return (_make_error_hash(106, $error_hash->{106}.$msg)) if ($err);
	    
	    $flag = 1;
	}
    }
    
    return (_make_error_hash(105, $error_hash->{105})) if (!$flag);
    
    return (_make_error_hash(0, $error_hash->{0}));
}

# This method permit to the user to shutdown an interface
# The parameter is an hash containing parameters.
sub ifdown
{
    my ($OBJ, $param_hash) = @_;

    # Testing the read only mode..
    return(_make_error_hash(200, $error_hash->{200})) if ($OBJ->{read_only});

    foreach my $tmp_hash (@{$param_hash->{'iface'}})
    {
	if (exists($tmp_hash->{'name'}))
	{
	    my ($err, $msg) = Lib_Network::ifdown($tmp_hash->{'name'}[0]);
	    return (_make_error_hash(104, $error_hash->{104}.$msg)) if ($err);
	}
    }

    return (_make_error_hash(0, $error_hash->{0}));
}

# This method permit to the user to start an interface
# The parameter is an hash containing parameters.
sub ifup
{
    my ($OBJ, $param_hash) = @_;

    # Testing the read only mode..
    return(_make_error_hash(200, $error_hash->{200})) if ($OBJ->{read_only});

    foreach my $tmp_hash (@{$param_hash->{'iface'}})
    {
	if (exists($tmp_hash->{'name'}))
	{
	    my ($err, $msg) = Lib_Network::ifup($tmp_hash->{'name'}[0]);
	    return (_make_error_hash(104, $error_hash->{104}.$msg)) if ($err);
	}
    }

    return (_make_error_hash(0, $error_hash->{0}));
}

# This method permit to the user to restart all interfaces
# There is no parameter to this method.
sub network_restart
{
    my ($OBJ) = @_;

    # Testing the read only mode..
    return(_make_error_hash(200, $error_hash->{200})) if ($OBJ->{read_only});

    my ($err, $msg) = Lib_Network::network_restart();

    return (_make_error_hash(104, $error_hash->{104}.$msg)) if ($err);

    return (_make_error_hash(0, $error_hash->{0}));
}

# This method permit to the user to get all available method
# There is no parameter.
sub list_methods
{
    my $OBJ = shift;

    my $hash_return = {
	'methods' => [
		      'list_code_error',
		      'get',
		      'info',
		      ],
	};
   
    push(@{$hash_return->{'methods'}}, "set") if(!$OBJ->{read_only});
    push(@{$hash_return->{'methods'}}, "ifdown") if(!$OBJ->{read_only});
    push(@{$hash_return->{'methods'}}, "ifup") if(!$OBJ->{read_only});
    push(@{$hash_return->{'methods'}}, "network_restart") if(!$OBJ->{read_only});
    push(@{$hash_return->{'methods'}}, "delete") if(!$OBJ->{read_only});

    return($hash_return);
}


# This method permit to the user to get all error codes.
# There is no parameter.
sub list_error_codes
{
    my $hash_return = {
	'error_codes' => []
    };

    foreach my $key (sort keys(%{$error_hash}))
    {
	my $tmp_hash = {
	    'error' => [$key],
	    'msg' => [$error_hash->{$key}],
	};
	push(@{$hash_return->{'error_codes'}}, $tmp_hash);
    }

    return($hash_return);
}

### Privates methods
sub _make_error_hash
{
    my ($error_code, $error_msg) = @_;

    my $hash_error = {
	'error_code' => [$error_code],
	'error_msg' => [$error_msg]
	};

    return($hash_error);
}

# Fonction un peu footwarIQ
# A refaire en recursif
sub _make_hash
{
    my ($hash, $name_key) = @_;
    
    my $hash_return = {};
    foreach my $key (sort(keys(%{$hash})))
    {
	my $tmp_hash = {};
	$tmp_hash->{'name'}[0] = $key;
	
	foreach my $param (sort(keys(%{$hash->{$key}})))
	{
	    if (ref($hash->{$key}->{$param}) eq "HASH")
	    {
		my $sub_hash = {};
		foreach my $sub_param (sort(keys(%{$hash->{$key}->{$param}})))
		{
		    $sub_hash->{$sub_param}[0] = $hash->{$key}->{$param}->{$sub_param};
		}
		$tmp_hash->{$param}[0] = $sub_hash;
	    }
	    else
	    {
		$tmp_hash->{$param}[0] = $hash->{$key}->{$param};
	    }
	}
	push @{$hash_return->{'iface'}}, $tmp_hash;
    }
    
    return ($hash_return);
}

sub _unmake_hash
{
    my ($hash) = @_;
    
    my $hash_return = {};

    foreach my $tmp_hash (@{$hash->{'iface'}})
    {
 	_unmake_subhash($hash_return, $tmp_hash);
    }
    
    return ($hash_return);
}

sub _unmake_subhash
{
    my ($hash_return, $hash) = @_;

    my $dev = $hash->{'name'}[0];

    foreach my $key (sort(keys(%{$hash})))
    {
 	if (ref($hash->{$key}[0]) eq 'HASH')
 	{
 	    foreach my $tmp_hash (@{$hash->{$key}})
 	    {
  		foreach my $sub_key (sort(keys(%{$tmp_hash})))
  		{
 		    $hash_return->{$dev}->{$key}->{$sub_key} = $tmp_hash->{$sub_key}[0];
  		}
 	    }
 	}
 	else
 	{
	    $hash_return->{$dev}->{$key} = $hash->{$key}[0] if ($key ne 'name');
 	}
    }

    return ($hash_return);
}

1;
